@@ -15,19 +15,19 @@ az.style.use("arviz-darkgrid")
 class ExperimentalDesign:
     """Base class"""
 
-    prediction_model = None
+    model = None
     expt_type = None
 
-    def __init__(self, prediction_model=None, **kwargs):
-        if prediction_model is not None:
-            self.prediction_model = prediction_model
-        if self.prediction_model is None:
+    def __init__(self, model=None, **kwargs):
+        if model is not None:
+            self.model = model
+        if self.model is None:
             raise ValueError("fitting_model not set or passed.")
 
     @property
     def idata(self):
         """Access to the InferenceData object"""
-        return self.prediction_model.idata
+        return self.model.idata
 
     def print_coefficients(self):
         """Prints the model coefficients"""
@@ -41,9 +41,7 @@ class ExperimentalDesign:
                 f"{name: <30}{coeff_samples.mean().data:.2f}, 94% HDI [{coeff_samples.quantile(0.03).data:.2f}, {coeff_samples.quantile(1-0.03).data:.2f}]"  # noqa: E501
             )
         # add coeff for measurement std
-        coeff_samples = az.extract(
-            self.prediction_model.idata.posterior, var_names="sigma"
-        )
+        coeff_samples = az.extract(self.model.idata.posterior, var_names="sigma")
         name = "sigma"
         print(
             f"{name: <30}{coeff_samples.mean().data:.2f}, 94% HDI [{coeff_samples.quantile(0.03).data:.2f}, {coeff_samples.quantile(1-0.03).data:.2f}]"  # noqa: E501
@@ -58,10 +56,10 @@ class TimeSeriesExperiment(ExperimentalDesign):
         data: pd.DataFrame,
         treatment_time: int,
         formula: str,
-        prediction_model=None,
+        model=None,
         **kwargs,
     ) -> None:
-        super().__init__(prediction_model=prediction_model, **kwargs)
+        super().__init__(model=model, **kwargs)
         self.treatment_time = treatment_time
         # split data in to pre and post intervention
         self.datapre = data[data.index <= self.treatment_time]
@@ -86,28 +84,28 @@ class TimeSeriesExperiment(ExperimentalDesign):
         # DEVIATION FROM SKL EXPERIMENT CODE =============================
         # fit the model to the observed (pre-intervention) data
         COORDS = {"coeffs": self.labels, "obs_indx": np.arange(self.pre_X.shape[0])}
-        self.prediction_model.fit(X=self.pre_X, y=self.pre_y, coords=COORDS)
+        self.model.fit(X=self.pre_X, y=self.pre_y, coords=COORDS)
         # ================================================================
 
         # score the goodness of fit to the pre-intervention data
-        self.score = self.prediction_model.score(X=self.pre_X, y=self.pre_y)
+        self.score = self.model.score(X=self.pre_X, y=self.pre_y)
 
         # get the model predictions of the observed (pre-intervention) data
-        self.pre_pred = self.prediction_model.predict(X=self.pre_X)
+        self.pre_pred = self.model.predict(X=self.pre_X)
 
         # calculate the counterfactual
-        self.post_pred = self.prediction_model.predict(X=self.post_X)
+        self.post_pred = self.model.predict(X=self.post_X)
 
         # causal impact pre (ie the residuals of the model fit to observed)
         pre_data = xr.DataArray(self.pre_y[:, 0], dims=["obs_ind"])
         self.pre_impact = (
-            pre_data - self.pre_pred["posterior_predictive"].y_hat
+            pre_data - self.pre_pred["posterior_predictive"].mu
         ).transpose(..., "obs_ind")
 
         # causal impact post (ie the residuals of the model fit to observed)
         post_data = xr.DataArray(self.post_y[:, 0], dims=["obs_ind"])
         self.post_impact = (
-            post_data - self.post_pred["posterior_predictive"].y_hat
+            post_data - self.post_pred["posterior_predictive"].mu
         ).transpose(..., "obs_ind")
 
         # cumulative impact post
@@ -120,31 +118,43 @@ class TimeSeriesExperiment(ExperimentalDesign):
 
         # TOP PLOT --------------------------------------------------
         # pre-intervention period
-        plot_xY(
+        h_line, h_patch = plot_xY(
             self.datapre.index,
-            self.pre_pred["posterior_predictive"].y_hat,
+            self.pre_pred["posterior_predictive"].mu,
             ax=ax[0],
+            plot_hdi_kwargs={"color": "C0"},
         )
-        ax[0].plot(self.datapre.index, self.pre_y, "k.", label="Observations")
+        handles = [(h_line, h_patch)]
+        labels = ["Pre-intervention period"]
+
+        (h,) = ax[0].plot(self.datapre.index, self.pre_y, "k.", label="Observations")
+        handles.append(h)
+        labels.append("Observations")
+
         # post intervention period
-        plot_xY(
+        h_line, h_patch = plot_xY(
             self.datapost.index,
-            self.post_pred["posterior_predictive"].y_hat,
+            self.post_pred["posterior_predictive"].mu,
             ax=ax[0],
-            include_label=False,
+            plot_hdi_kwargs={"color": "C1"},
         )
+        handles.append((h_line, h_patch))
+        labels.append("Synthetic control")
+
         ax[0].plot(self.datapost.index, self.post_y, "k.")
         # Shaded causal effect
-        ax[0].fill_between(
+        h = ax[0].fill_between(
             self.datapost.index,
             y1=az.extract(
-                self.post_pred, group="posterior_predictive", var_names="y_hat"
+                self.post_pred, group="posterior_predictive", var_names="mu"
             ).mean("sample"),
             y2=np.squeeze(self.post_y),
             color="C0",
             alpha=0.25,
-            label="Causal impact",
         )
+        handles.append(h)
+        labels.append("Causal impact")
+
         ax[0].set(
             title=f"""
             Pre-intervention Bayesian $R^2$: {self.score.r2:.3f}
@@ -157,12 +167,13 @@ class TimeSeriesExperiment(ExperimentalDesign):
             self.datapre.index,
             self.pre_impact,
             ax=ax[1],
+            plot_hdi_kwargs={"color": "C0"},
         )
         plot_xY(
             self.datapost.index,
             self.post_impact,
             ax=ax[1],
-            include_label=False,
+            plot_hdi_kwargs={"color": "C1"},
         )
         ax[1].axhline(y=0, c="k")
         ax[1].fill_between(
@@ -175,12 +186,12 @@ class TimeSeriesExperiment(ExperimentalDesign):
         ax[1].set(title="Causal Impact")
 
         # BOTTOM PLOT -----------------------------------------------
-
         ax[2].set(title="Cumulative Causal Impact")
         plot_xY(
             self.datapost.index,
             self.post_impact_cumulative,
             ax=ax[2],
+            plot_hdi_kwargs={"color": "C1"},
         )
         ax[2].axhline(y=0, c="k")
 
@@ -191,10 +202,13 @@ class TimeSeriesExperiment(ExperimentalDesign):
                 ls="-",
                 lw=3,
                 color="r",
-                label="Treatment time",
             )
 
-        ax[0].legend(fontsize=LEGEND_FONT_SIZE)
+        ax[0].legend(
+            handles=(h_tuple for h_tuple in handles),
+            labels=labels,
+            fontsize=LEGEND_FONT_SIZE,
+        )
 
         return (fig, ax)
 
@@ -242,10 +256,10 @@ class DifferenceInDifferences(ExperimentalDesign):
         group_variable_name: str,
         treated: str,
         untreated: str,
-        prediction_model=None,
+        model=None,
         **kwargs,
     ):
-        super().__init__(prediction_model=prediction_model, **kwargs)
+        super().__init__(model=model, **kwargs)
         self.data = data
         self.expt_type = "Difference in Differences"
         self.formula = formula
@@ -265,11 +279,11 @@ class DifferenceInDifferences(ExperimentalDesign):
         # Input validation ----------------------------------------------------
         # Check that `treated` appears in the module formula
         assert (
-            "treated" in formula
-        ), "A predictor column called `treated` should be in the provided dataframe"
+            "post_treatment" in formula
+        ), "A predictor called `post_treatment` should be in the dataframe"
         # Check that we have `treated` in the incoming dataframe
         assert (
-            "treated" in self.data.columns
+            "post_treatment" in self.data.columns
         ), "Require a boolean column labelling observations which are `treated`"
         # Check for `unit` in the incoming dataframe.
         # *This is only used for plotting purposes*
@@ -289,138 +303,157 @@ class DifferenceInDifferences(ExperimentalDesign):
             .I.e. the treated and untreated.
         """
 
-        # TODO: `treated` is a deterministic function of group and time, so this could
-        # be a function rather than supplied data
-
         # DEVIATION FROM SKL EXPERIMENT CODE =============================
-        # fit the model to the observed (pre-intervention) data
         COORDS = {"coeffs": self.labels, "obs_indx": np.arange(self.X.shape[0])}
-        self.prediction_model.fit(X=self.X, y=self.y, coords=COORDS)
+        self.model.fit(X=self.X, y=self.y, coords=COORDS)
         # ================================================================
 
-        time_levels = self.data[self.time_variable_name].unique()
-
         # predicted outcome for control group
-        self.x_pred_control = pd.DataFrame(
-            {
-                self.group_variable_name: [self.untreated, self.untreated],
-                self.time_variable_name: time_levels,
-                "treated": [0, 0],
-            }
+        self.x_pred_control = (
+            self.data
+            # just the untreated group
+            .query(f"{self.group_variable_name} == @self.untreated")
+            # drop the outcome variable
+            .drop(self.outcome_variable_name, axis=1)
+            # We may have multiple units per time point, we only want one time point
+            .groupby(self.time_variable_name)
+            .first()
+            .reset_index()
         )
+        assert not self.x_pred_control.empty
         (new_x,) = build_design_matrices([self._x_design_info], self.x_pred_control)
-        self.y_pred_control = self.prediction_model.predict(np.asarray(new_x))
+        self.y_pred_control = self.model.predict(np.asarray(new_x))
 
         # predicted outcome for treatment group
-        self.x_pred_treatment = pd.DataFrame(
-            {
-                self.group_variable_name: [self.treated, self.treated],
-                self.time_variable_name: time_levels,
-                "treated": [0, 1],
-            }
+        self.x_pred_treatment = (
+            self.data
+            # just the treated group
+            .query(f"{self.group_variable_name} == @self.treated")
+            # drop the outcome variable
+            .drop(self.outcome_variable_name, axis=1)
+            # We may have multiple units per time point, we only want one time point
+            .groupby(self.time_variable_name)
+            .first()
+            .reset_index()
         )
+        assert not self.x_pred_treatment.empty
         (new_x,) = build_design_matrices([self._x_design_info], self.x_pred_treatment)
-        self.y_pred_treatment = self.prediction_model.predict(np.asarray(new_x))
+        self.y_pred_treatment = self.model.predict(np.asarray(new_x))
 
         # predicted outcome for counterfactual
-        self.x_pred_counterfactual = pd.DataFrame(
-            {
-                self.group_variable_name: [self.treated],
-                self.time_variable_name: time_levels[1],
-                "treated": [0],
-            }
+        self.x_pred_counterfactual = (
+            self.data
+            # just the treated group
+            .query(f"{self.group_variable_name} == @self.treated")
+            # just the treatment period(s)
+            .query("post_treatment == True")
+            # drop the outcome variable
+            .drop(self.outcome_variable_name, axis=1)
+            # DO AN INTERVENTION. Set the post_treatment variable to False
+            .assign(post_treatment=False)
+            # We may have multiple units per time point, we only want one time point
+            .groupby(self.time_variable_name)
+            .first()
+            .reset_index()
         )
+        assert not self.x_pred_counterfactual.empty
         (new_x,) = build_design_matrices(
             [self._x_design_info], self.x_pred_counterfactual
         )
-        self.y_pred_counterfactual = self.prediction_model.predict(np.asarray(new_x))
+        self.y_pred_counterfactual = self.model.predict(np.asarray(new_x))
 
         # calculate causal impact
         self.causal_impact = (
             self.y_pred_treatment["posterior_predictive"].mu.isel({"obs_ind": 1})
             - self.y_pred_counterfactual["posterior_predictive"].mu.squeeze()
         )
-        # self.causal_impact = (
-        #     self.y_pred_treatment["posterior_predictive"]
-        #     .mu.isel({"obs_ind": 1})
-        #     .stack(samples=["chain", "draw"])
-        #     - self.y_pred_counterfactual["posterior_predictive"]
-        #     .mu.stack(samples=["chain", "draw"])
-        #     .squeeze()
-        # )
 
     def plot(self):
-        """Plot the results"""
+        """Plot the results.
+        Creating the combined mean + HDI legend entries is a bit involved.
+        """
         fig, ax = plt.subplots()
 
         # Plot raw data
-        # NOTE: This will not work when there is just ONE unit in each group
-        sns.lineplot(
+        sns.scatterplot(
             self.data,
             x=self.time_variable_name,
             y=self.outcome_variable_name,
             hue=self.group_variable_name,
-            units="unit",  # NOTE: assumes we have a `unit` predictor variable
-            estimator=None,
-            alpha=0.5,
+            alpha=1,
+            legend=False,
+            markers=True,
             ax=ax,
         )
+
         # Plot model fit to control group
-        parts = ax.violinplot(
-            az.extract(
-                self.y_pred_control, group="posterior_predictive", var_names="mu"
-            ).values.T,
-            positions=self.x_pred_control[self.time_variable_name].values,
-            showmeans=False,
-            showmedians=False,
-            widths=0.2,
+        time_points = self.x_pred_control[self.time_variable_name].values
+        h_line, h_patch = plot_xY(
+            time_points,
+            self.y_pred_control.posterior_predictive.mu,
+            ax=ax,
+            plot_hdi_kwargs={"color": "C0"},
+            label="Control group",
         )
-        for pc in parts["bodies"]:
-            pc.set_facecolor("C0")
-            pc.set_edgecolor("None")
-            pc.set_alpha(0.5)
+        handles = [(h_line, h_patch)]
+        labels = ["Control group"]
 
         # Plot model fit to treatment group
-        parts = ax.violinplot(
-            az.extract(
-                self.y_pred_treatment, group="posterior_predictive", var_names="mu"
-            ).values.T,
-            positions=self.x_pred_treatment[self.time_variable_name].values,
-            showmeans=False,
-            showmedians=False,
-            widths=0.2,
+        time_points = self.x_pred_control[self.time_variable_name].values
+        h_line, h_patch = plot_xY(
+            time_points,
+            self.y_pred_treatment.posterior_predictive.mu,
+            ax=ax,
+            plot_hdi_kwargs={"color": "C1"},
+            label="Treatment group",
         )
+        handles.append((h_line, h_patch))
+        labels.append("Treatment group")
 
-        for pc in parts["bodies"]:
-            pc.set_facecolor("C1")
-            pc.set_edgecolor("None")
-            pc.set_alpha(0.5)
         # Plot counterfactual - post-test for treatment group IF no treatment
         # had occurred.
-        parts = ax.violinplot(
-            az.extract(
-                self.y_pred_counterfactual,
-                group="posterior_predictive",
-                var_names="mu",
-            ).values.T,
-            positions=self.x_pred_counterfactual[self.time_variable_name].values,
-            showmeans=False,
-            showmedians=False,
-            widths=0.2,
-        )
-        for pc in parts["bodies"]:
-            pc.set_facecolor("C2")
-            pc.set_edgecolor("None")
-            pc.set_alpha(0.5)
+        time_points = self.x_pred_counterfactual[self.time_variable_name].values
+        if len(time_points) == 1:
+            parts = ax.violinplot(
+                az.extract(
+                    self.y_pred_counterfactual,
+                    group="posterior_predictive",
+                    var_names="mu",
+                ).values.T,
+                positions=self.x_pred_counterfactual[self.time_variable_name].values,
+                showmeans=False,
+                showmedians=False,
+                widths=0.2,
+            )
+            for pc in parts["bodies"]:
+                pc.set_facecolor("C0")
+                pc.set_edgecolor("None")
+                pc.set_alpha(0.5)
+        else:
+            h_line, h_patch = plot_xY(
+                time_points,
+                self.y_pred_counterfactual.posterior_predictive.mu,
+                ax=ax,
+                plot_hdi_kwargs={"color": "C2"},
+                label="Counterfactual",
+            )
+            handles.append((h_line, h_patch))
+            labels.append("Counterfactual")
+
         # arrow to label the causal impact
         self._plot_causal_impact_arrow(ax)
+
         # formatting
         ax.set(
             xticks=self.x_pred_treatment[self.time_variable_name].values,
             title=self._causal_impact_summary_stat(),
         )
-        ax.legend(fontsize=LEGEND_FONT_SIZE)
-        return (fig, ax)
+        ax.legend(
+            handles=(h_tuple for h_tuple in handles),
+            labels=labels,
+            fontsize=LEGEND_FONT_SIZE,
+        )
+        return fig, ax
 
     def _plot_causal_impact_arrow(self, ax):
         """
@@ -485,7 +518,7 @@ class RegressionDiscontinuity(ExperimentalDesign):
     :param formula: A statistical model formula
     :param treatment_threshold: A scalar threshold value at which the treatment
                                 is applied
-    :param prediction_model: A PyMC model
+    :param model: A PyMC model
     :param running_variable_name: The name of the predictor variable that the treatment
                                   threshold is based upon
 
@@ -500,11 +533,11 @@ class RegressionDiscontinuity(ExperimentalDesign):
         data: pd.DataFrame,
         formula: str,
         treatment_threshold: float,
-        prediction_model=None,
+        model=None,
         running_variable_name: str = "x",
         **kwargs,
     ):
-        super().__init__(prediction_model=prediction_model, **kwargs)
+        super().__init__(model=model, **kwargs)
         self.expt_type = "Regression Discontinuity"
         self.data = data
         self.formula = formula
@@ -523,11 +556,11 @@ class RegressionDiscontinuity(ExperimentalDesign):
         # DEVIATION FROM SKL EXPERIMENT CODE =============================
         # fit the model to the observed (pre-intervention) data
         COORDS = {"coeffs": self.labels, "obs_indx": np.arange(self.X.shape[0])}
-        self.prediction_model.fit(X=self.X, y=self.y, coords=COORDS)
+        self.model.fit(X=self.X, y=self.y, coords=COORDS)
         # ================================================================
 
         # score the goodness of fit to all data
-        self.score = self.prediction_model.score(X=self.X, y=self.y)
+        self.score = self.model.score(X=self.X, y=self.y)
 
         # get the model predictions of the observed data
         xi = np.linspace(
@@ -539,7 +572,7 @@ class RegressionDiscontinuity(ExperimentalDesign):
             {self.running_variable_name: xi, "treated": self._is_treated(xi)}
         )
         (new_x,) = build_design_matrices([self._x_design_info], self.x_pred)
-        self.pred = self.prediction_model.predict(X=np.asarray(new_x))
+        self.pred = self.model.predict(X=np.asarray(new_x))
 
         # calculate discontinuity by evaluating the difference in model expectation on
         # either side of the discontinuity
@@ -554,7 +587,7 @@ class RegressionDiscontinuity(ExperimentalDesign):
             }
         )
         (new_x,) = build_design_matrices([self._x_design_info], self.x_discon)
-        self.pred_discon = self.prediction_model.predict(X=np.asarray(new_x))
+        self.pred_discon = self.model.predict(X=np.asarray(new_x))
         self.discontinuity_at_threshold = (
             self.pred_discon["posterior_predictive"].sel(obs_ind=1)["mu"]
             - self.pred_discon["posterior_predictive"].sel(obs_ind=0)["mu"]
@@ -580,12 +613,17 @@ class RegressionDiscontinuity(ExperimentalDesign):
             c="k",  # hue="treated",
             ax=ax,
         )
+
         # Plot model fit to data
-        plot_xY(
+        h_line, h_patch = plot_xY(
             self.x_pred[self.running_variable_name],
             self.pred["posterior_predictive"].mu,
             ax=ax,
+            plot_hdi_kwargs={"color": "C1"},
         )
+        handles = [(h_line, h_patch)]
+        labels = ["Posterior mean"]
+
         # create strings to compose title
         title_info = f"{self.score.r2:.3f} (std = {self.score.r2_std:.3f})"
         r2 = f"Bayesian $R^2$ on all data = {title_info}"
@@ -603,7 +641,11 @@ class RegressionDiscontinuity(ExperimentalDesign):
             color="r",
             label="treatment threshold",
         )
-        ax.legend(fontsize=LEGEND_FONT_SIZE)
+        ax.legend(
+            handles=(h_tuple for h_tuple in handles),
+            labels=labels,
+            fontsize=LEGEND_FONT_SIZE,
+        )
         return (fig, ax)
 
     def summary(self):
@@ -629,10 +671,10 @@ class PrePostNEGD(ExperimentalDesign):
         formula: str,
         group_variable_name: str,
         pretreatment_variable_name: str,
-        prediction_model=None,
+        model=None,
         **kwargs,
     ):
-        super().__init__(prediction_model=prediction_model, **kwargs)
+        super().__init__(model=model, **kwargs)
         self.data = data
         self.expt_type = "Pretest/posttest Nonequivalent Group Design"
         self.formula = formula
@@ -659,7 +701,7 @@ class PrePostNEGD(ExperimentalDesign):
 
         # fit the model to the observed (pre-intervention) data
         COORDS = {"coeffs": self.labels, "obs_indx": np.arange(self.X.shape[0])}
-        self.prediction_model.fit(X=self.X, y=self.y, coords=COORDS)
+        self.model.fit(X=self.X, y=self.y, coords=COORDS)
 
         # Calculate the posterior predictive for the treatment and control for an
         # interpolated set of pretest values
@@ -677,7 +719,7 @@ class PrePostNEGD(ExperimentalDesign):
             }
         )
         (new_x,) = build_design_matrices([self._x_design_info], x_pred_untreated)
-        self.pred_untreated = self.prediction_model.predict(X=np.asarray(new_x))
+        self.pred_untreated = self.model.predict(X=np.asarray(new_x))
         # treated
         x_pred_untreated = pd.DataFrame(
             {
@@ -686,7 +728,7 @@ class PrePostNEGD(ExperimentalDesign):
             }
         )
         (new_x,) = build_design_matrices([self._x_design_info], x_pred_untreated)
-        self.pred_treated = self.prediction_model.predict(X=np.asarray(new_x))
+        self.pred_treated = self.model.predict(X=np.asarray(new_x))
 
         # Evaluate causal impact as equal to the trestment effect
         self.causal_impact = self.idata.posterior["beta"].sel(
@@ -708,27 +750,38 @@ class PrePostNEGD(ExperimentalDesign):
             hue="group",
             alpha=0.5,
             data=self.data,
+            legend=True,
             ax=ax[0],
         )
         ax[0].set(xlabel="Pretest", ylabel="Posttest")
 
         # plot posterior predictive of untreated
-        plot_xY(
+        h_line, h_patch = plot_xY(
             self.pred_xi,
-            self.pred_untreated["posterior_predictive"].y_hat,
+            self.pred_untreated["posterior_predictive"].mu,
             ax=ax[0],
             plot_hdi_kwargs={"color": "C0"},
+            label="Control group",
         )
+        handles = [(h_line, h_patch)]
+        labels = ["Control group"]
 
         # plot posterior predictive of treated
-        plot_xY(
+        h_line, h_patch = plot_xY(
             self.pred_xi,
-            self.pred_treated["posterior_predictive"].y_hat,
+            self.pred_treated["posterior_predictive"].mu,
             ax=ax[0],
             plot_hdi_kwargs={"color": "C1"},
+            label="Treatment group",
         )
+        handles.append((h_line, h_patch))
+        labels.append("Treatment group")
 
-        ax[0].legend(fontsize=LEGEND_FONT_SIZE)
+        ax[0].legend(
+            handles=(h_tuple for h_tuple in handles),
+            labels=labels,
+            fontsize=LEGEND_FONT_SIZE,
+        )
 
         # Plot estimated caual impact / treatment effect
         az.plot_posterior(self.causal_impact, ref_val=0, ax=ax[1])
